@@ -417,16 +417,22 @@ fsg_model_read(FILE * fp, logmath_t *lmath, float32 lw)
 
         if ((strcmp(wordptr[0], FSG_MODEL_BEGIN_DECL) == 0)) {
             if (n > 2) {
-                E_ERROR("Line[%d]: malformed FSG_BEGIN delcaration\n",
+                E_ERROR("Line[%d]: malformed FSG_BEGIN declaration\n",
                         lineno);
                 goto parse_error;
             }
             break;
         }
     }
-    /* Save FSG name, or it will get clobbered below :( */
-    if (n == 2)
+    /* Save FSG name, or it will get clobbered below :(.
+     * If name is missing, try the default.
+     */
+    if (n == 2) {
         fsgname = ckd_salloc(wordptr[1]);
+    } else {
+        E_WARN ("FSG name is missing\n");
+        fsgname = ckd_salloc("unknown");
+    }
 
     /* Read #states */
     n = nextline_str2words(fp, &lineno, &lineptr, &wordptr);
@@ -493,18 +499,26 @@ fsg_model_read(FILE * fp, logmath_t *lmath, float32 lw)
 
         if ((strcmp(wordptr[0], FSG_MODEL_T_DECL) == 0)
             || (strcmp(wordptr[0], FSG_MODEL_TRANSITION_DECL) == 0)) {
+
+
             if (((n != 4) && (n != 5))
                 || (sscanf(wordptr[1], "%d", &i) != 1)
                 || (sscanf(wordptr[2], "%d", &j) != 1)
-                || (sscanf(wordptr[3], "%f", &p) != 1)
                 || (i < 0) || (i >= fsg->n_state)
-                || (j < 0) || (j >= fsg->n_state)
-                || (p <= 0.0) || (p > 1.0)) {
+                || (j < 0) || (j >= fsg->n_state)) {
                 E_ERROR
                     ("Line[%d]: transition spec malformed; Expecting: from-state to-state trans-prob [word]\n",
                      lineno);
                 goto parse_error;
             }
+
+            p = atof_c(wordptr[3]);
+            if ((p <= 0.0) || (p > 1.0)) {
+                E_ERROR
+                    ("Line[%d]: transition spec malformed; Expecting float as transition probability\n",
+                     lineno);
+                goto parse_error;
+            }
         }
         else {
             E_ERROR("Line[%d]: transition or FSG_END statement expected\n",
@@ -623,7 +637,7 @@ fsg_model_write(fsg_model_t * fsg, FILE * fp)
     gnode_t *gn;
     fsg_link_t *tl;
 
-    fprintf(fp, "%s\n", FSG_MODEL_BEGIN_DECL);
+    fprintf(fp, "%s %s\n", FSG_MODEL_BEGIN_DECL, fsg->name ? fsg->name : "");
     fprintf(fp, "%s %d\n", FSG_MODEL_NUM_STATES_DECL, fsg->n_state);
     fprintf(fp, "%s %d\n", FSG_MODEL_START_STATE_DECL, fsg->start_state);
     fprintf(fp, "%s %d\n", FSG_MODEL_FINAL_STATE_DECL, fsg->final_state);
@@ -651,7 +665,6 @@ fsg_model_write(fsg_model_t * fsg, FILE * fp)
         }
     }
 
-    fprintf(fp, "%c\n", FSG_MODEL_COMMENT_CHAR);
     fprintf(fp, "%s\n", FSG_MODEL_END_DECL);
 
     fflush(fp);
@@ -675,3 +688,103 @@ fsg_model_writefile(fsg_model_t *fsg, char const *file)
 
     fclose(fp);
 }
+
+static void
+fsg_model_write_fsm_trans(fsg_model_t *fsg, int i, FILE *fp)
+{
+    int j;
+
+    for (j = 0; j < fsg->n_state; j++) {
+        gnode_t *gn;
+        fsg_link_t *tl;
+
+        /* Print non-null transitions */
+        for (gn = fsg->trans[i][j]; gn; gn = gnode_next(gn)) {
+            tl = (fsg_link_t *) gnode_ptr(gn);
+
+            fprintf(fp, "%d %d %s %f\n",
+                    tl->from_state, tl->to_state,
+                    (tl->wid < 0) ? "<eps>" : fsg_model_word_str(fsg, tl->wid),
+                    -logmath_log_to_ln(fsg->lmath, tl->logs2prob / fsg->lw));
+        }
+
+        /* Print null transitions */
+        tl = fsg->null_trans[i][j];
+        if (tl) {
+            fprintf(fp, "%d %d <eps> %f\n",
+                    tl->from_state, tl->to_state,
+                    -logmath_log_to_ln(fsg->lmath, tl->logs2prob / fsg->lw));
+        }
+    }
+}
+
+void
+fsg_model_write_fsm(fsg_model_t * fsg, FILE * fp)
+{
+    int i;
+
+    /* Write transitions from initial state first. */
+    fsg_model_write_fsm_trans(fsg, fsg_model_start_state(fsg), fp);
+
+    /* Other states. */
+    for (i = 0; i < fsg->n_state; i++) {
+        if (i == fsg_model_start_state(fsg))
+            continue;
+        fsg_model_write_fsm_trans(fsg, i, fp);
+    }
+
+    /* Final state. */
+    fprintf(fp, "%d 0\n", fsg_model_final_state(fsg));
+
+    fflush(fp);
+}
+
+void
+fsg_model_writefile_fsm(fsg_model_t *fsg, char const *file)
+{
+    FILE *fp;
+
+    assert(fsg);
+
+    E_INFO("Writing FSM file '%s'\n", file);
+
+    if ((fp = fopen(file, "w")) == NULL) {
+        E_ERROR("fopen(%s,r) failed\n", file);
+        return;
+    }
+
+    fsg_model_write_fsm(fsg, fp);
+
+    fclose(fp);
+}
+
+void
+fsg_model_write_symtab(fsg_model_t *fsg, FILE *file)
+{
+    int i;
+
+    fprintf(file, "<eps> 0\n");
+    for (i = 0; i < fsg_model_n_word(fsg); ++i) {
+        fprintf(file, "%s %d\n", fsg_model_word_str(fsg, i), i + 1);
+    }
+    fflush(file);
+}
+
+void
+fsg_model_writefile_symtab(fsg_model_t *fsg, char const *file)
+{
+    FILE *fp;
+
+    assert(fsg);
+
+    E_INFO("Writing FSM symbol table '%s'\n", file);
+
+    if ((fp = fopen(file, "w")) == NULL) {
+        E_ERROR("fopen(%s,r) failed\n", file);
+        return;
+    }
+
+    fsg_model_write_symtab(fsg, fp);
+
+    fclose(fp);
+}
