@@ -96,6 +96,9 @@ jsgf_grammar_new(jsgf_t *parent)
 
         grammar->rules = hash_table_new(64, 0);
         grammar->imports = hash_table_new(16, 0);
+
+        /* Silvio Moioli: no getenv() in Windows CE */
+        #if !defined(_WIN32_WCE)
         if ((jsgf_path = getenv("JSGF_PATH")) != NULL) {
             char *word, *c;
 
@@ -114,6 +117,7 @@ jsgf_grammar_new(jsgf_t *parent)
             /* Default to current directory. */
             grammar->searchpath = glist_add_ptr(grammar->searchpath, ckd_salloc("."));
         }
+        #endif 
     }
 
     return grammar;
@@ -437,8 +441,9 @@ jsgf_rule_public(jsgf_rule_t *rule)
     return rule->public;
 }
 
-fsg_model_t *
-jsgf_build_fsg(jsgf_t *grammar, jsgf_rule_t *rule, logmath_t *lmath, float32 lw)
+static fsg_model_t *
+jsgf_build_fsg_internal(jsgf_t *grammar, jsgf_rule_t *rule,
+                        logmath_t *lmath, float32 lw, int do_closure)
 {
     fsg_model_t *fsg;
     glist_t nulls;
@@ -476,60 +481,45 @@ jsgf_build_fsg(jsgf_t *grammar, jsgf_rule_t *rule, logmath_t *lmath, float32 lw)
             fsg_model_null_trans_add(fsg, link->from, link->to, 0);
         }            
     }
-    nulls = fsg_model_null_trans_closure(fsg, NULL);
-    glist_free(nulls);
+    if (do_closure) {
+        nulls = fsg_model_null_trans_closure(fsg, NULL);
+        glist_free(nulls);
+    }
 
     return fsg;
 }
 
+fsg_model_t *
+jsgf_build_fsg(jsgf_t *grammar, jsgf_rule_t *rule,
+               logmath_t *lmath, float32 lw)
+{
+    return jsgf_build_fsg_internal(grammar, rule, lmath, lw, TRUE);
+}
+
+fsg_model_t *
+jsgf_build_fsg_raw(jsgf_t *grammar, jsgf_rule_t *rule,
+                   logmath_t *lmath, float32 lw)
+{
+    return jsgf_build_fsg_internal(grammar, rule, lmath, lw, FALSE);
+}
+
 int
 jsgf_write_fsg(jsgf_t *grammar, jsgf_rule_t *rule, FILE *outfh)
 {
-    gnode_t *gn;
+    fsg_model_t *fsg;
+    logmath_t *lmath = logmath_init(1.0001, 0, 0);
 
-    /* Clear previous links */
-    for (gn = grammar->links; gn; gn = gnode_next(gn)) {
-        ckd_free(gnode_ptr(gn));
-    }
-    glist_free(grammar->links);
-    grammar->links = NULL;
-    rule->entry = rule->exit = 0;
-    grammar->nstate = 0;
-
-    expand_rule(grammar, rule);
-
-    printf("FSG_BEGIN %s\n", rule->name);
-    printf("NUM_STATES %d\n", grammar->nstate);
-    printf("START_STATE %d\n", rule->entry);
-    printf("FINAL_STATE %d\n", rule->exit);
-    printf("\n# Transitions\n");
-
-    grammar->links = glist_reverse(grammar->links);
-    for (gn = grammar->links; gn; gn = gnode_next(gn)) {
-        jsgf_link_t *link = gnode_ptr(gn);
-
-        if (link->atom) {
-            if (jsgf_atom_is_rule(link->atom)) {
-                printf("TRANSITION %d %d %f\n",
-                       link->from, link->to,
-                       link->atom->weight);
-            }
-            else {
-                printf("TRANSITION %d %d %f %s\n",
-                       link->from, link->to,
-                       link->atom->weight, link->atom->name);
-            }
-        }
-        else {
-            printf("TRANSITION %d %d %f\n", link->from, link->to, 1.0);
-        }               
-    }
-
-    printf("FSG_END\n");
+    if ((fsg = jsgf_build_fsg_raw(grammar, rule, lmath, 1.0)) == NULL)
+        goto error_out;
 
+    fsg_model_write(fsg, outfh);
+    logmath_free(lmath);
     return 0;
-}
 
+error_out:
+    logmath_free(lmath);
+    return -1;
+}
 jsgf_rule_t *
 jsgf_define_rule(jsgf_t *jsgf, char *name, jsgf_rhs_t *rhs, int public)
 {
