@@ -64,104 +64,136 @@ ngram_file_name_to_type(const char *file_name)
 
     ext = strrchr(file_name, '.');
     if (ext == NULL) {
-        return NGRAM_ARPA; /* Default file type */
+        return NGRAM_INVALID;
     }
     if (0 == strcmp_nocase(ext, ".gz")) {
         while (--ext >= file_name) {
             if (*ext == '.') break;
         }
         if (ext < file_name) {
-            return NGRAM_ARPA; /* Default file type */
-        }
-    }
-    /* We use strncmp because there might be a .gz on the end. */
-    if (0 == strncmp_nocase(ext, ".ARPA", 5))
+            return NGRAM_INVALID;
+         }
+     }
+     else if (0 == strcmp_nocase(ext, ".bz2")) {
+         while (--ext >= file_name) {
+             if (*ext == '.') break;
+         }
+         if (ext < file_name) {
+             return NGRAM_INVALID;
+         }
+     }
+     /* We use strncmp because there might be a .gz on the end. */
+     if (0 == strncmp_nocase(ext, ".ARPA", 5))
+         return NGRAM_ARPA;
+     if (0 == strncmp_nocase(ext, ".DMP", 4))
+         return NGRAM_DMP;
+     return NGRAM_INVALID;
+ }
+
+ngram_file_type_t
+ngram_str_to_type(const char *str_name)
+{
+    if (0 == strcmp_nocase(str_name, "arpa"))
         return NGRAM_ARPA;
-    if (0 == strncmp_nocase(ext, ".DMP32", 6))
-        return NGRAM_DMP32;
-    if (0 == strncmp_nocase(ext, ".DMP", 4))
+    if (0 == strcmp_nocase(str_name, "dmp"))
         return NGRAM_DMP;
-    return NGRAM_ARPA; /* Default file type */
+    return NGRAM_INVALID;
 }
 
-ngram_model_t *
-ngram_model_read(cmd_ln_t *config,
-		 const char *file_name,
-                 ngram_file_type_t file_type,
-		 logmath_t *lmath)
+char const *
+ngram_type_to_str(int type)
 {
-    ngram_model_t *model = NULL;
-
-    switch (file_type) {
-    case NGRAM_AUTO: {
-        if ((model = ngram_model_arpa_read(config, file_name, lmath)) != NULL)
-            break;
-        if ((model = ngram_model_dmp_read(config, file_name, lmath)) != NULL)
-            break;
-        if ((model = ngram_model_dmp32_read(config, file_name, lmath)) != NULL)
-            break;
+    switch (type) {
+    case NGRAM_ARPA:
+        return "arpa";
+    case NGRAM_DMP:
+        return "dmp";
+    default:
         return NULL;
     }
-    case NGRAM_ARPA:
-        model = ngram_model_arpa_read(config, file_name, lmath);
-        break;
-    case NGRAM_DMP:
-        model = ngram_model_dmp_read(config, file_name, lmath);
-        break;
-    case NGRAM_DMP32:
-        model = ngram_model_dmp32_read(config, file_name, lmath);
-        break;
-    }
-
-    /* Now set weights based on config if present. */
-    if (config) {
-        float32 lw = 1.0;
-        float32 wip = 1.0;
-        float32 uw = 1.0;
-
-        if (cmd_ln_exists_r(config, "-lw"))
-            lw = cmd_ln_float32_r(config, "-lw");
-        if (cmd_ln_exists_r(config, "-wip"))
-            wip = cmd_ln_float32_r(config, "-wip");
-        if (cmd_ln_exists_r(config, "-uw"))
-            uw = cmd_ln_float32_r(config, "-uw");
-
-        ngram_model_apply_weights(model, lw, wip, uw);
-    }
-
-    return model;
 }
 
-int
-ngram_model_write(ngram_model_t *model, const char *file_name,
-		  ngram_file_type_t file_type)
-{
-    switch (file_type) {
-    case NGRAM_AUTO: {
-        file_type = ngram_file_name_to_type(file_name);
-        return ngram_model_write(model, file_name, file_type);
-    }
-    case NGRAM_ARPA:
-        return ngram_model_arpa_write(model, file_name);
-    case NGRAM_DMP:
-        return ngram_model_dmp_write(model, file_name);
-    case NGRAM_DMP32:
-        return ngram_model_dmp32_write(model, file_name);
-    }
 
-    return -1; /* In case your compiler is really stupid. */
-}
+ ngram_model_t *
+ ngram_model_read(cmd_ln_t *config,
+                  const char *file_name,
+                  ngram_file_type_t file_type,
+                  logmath_t *lmath)
+ {
+     ngram_model_t *model = NULL;
 
-int32
-ngram_model_init(ngram_model_t *base,
-                 ngram_funcs_t *funcs,
-                 logmath_t *lmath,
-                 int32 n, int32 n_unigram)
-{
-    base->refcount = 1;
-    base->funcs = funcs;
-    base->n = n;
-    /* If this was previously initialized... */
+     switch (file_type) {
+     case NGRAM_AUTO: {
+         if ((model = ngram_model_arpa_read(config, file_name, lmath)) != NULL)
+             break;
+         if ((model = ngram_model_dmp_read(config, file_name, lmath)) != NULL)
+             break;
+         return NULL;
+     }
+     case NGRAM_ARPA:
+         model = ngram_model_arpa_read(config, file_name, lmath);
+         break;
+     case NGRAM_DMP:
+         model = ngram_model_dmp_read(config, file_name, lmath);
+         break;
+     default:
+         E_ERROR("language model file type not supported\n");
+         return NULL;
+     }
+
+     /* Now set weights based on config if present. */
+     if (config) {
+         float32 lw = 1.0;
+         float32 wip = 1.0;
+         float32 uw = 1.0;
+
+         if (cmd_ln_exists_r(config, "-lw"))
+             lw = cmd_ln_float32_r(config, "-lw");
+         if (cmd_ln_exists_r(config, "-wip"))
+             wip = cmd_ln_float32_r(config, "-wip");
+         if (cmd_ln_exists_r(config, "-uw"))
+             uw = cmd_ln_float32_r(config, "-uw");
+
+         ngram_model_apply_weights(model, lw, wip, uw);
+     }
+
+     return model;
+ }
+
+ int
+ ngram_model_write(ngram_model_t *model, const char *file_name,
+                   ngram_file_type_t file_type)
+ {
+     switch (file_type) {
+     case NGRAM_AUTO: {
+         file_type = ngram_file_name_to_type(file_name);
+         /* Default to ARPA (catches .lm and other things) */
+         if (file_type == NGRAM_INVALID)
+             file_type = NGRAM_ARPA;
+         return ngram_model_write(model, file_name, file_type);
+     }
+     case NGRAM_ARPA:
+         return ngram_model_arpa_write(model, file_name);
+     case NGRAM_DMP:
+         return ngram_model_dmp_write(model, file_name);
+     default:
+         E_ERROR("language model file type not supported\n");
+         return -1;
+     }
+     E_ERROR("language model file type not supported\n");
+     return -1;
+ }
+
+ int32
+ ngram_model_init(ngram_model_t *base,
+                  ngram_funcs_t *funcs,
+                  logmath_t *lmath,
+                  int32 n, int32 n_unigram)
+ {
+     base->refcount = 1;
+     base->funcs = funcs;
+     base->n = n;
+     /* If this was previously initialized... */
     if (base->n_counts == NULL)
         base->n_counts = ckd_calloc(3, sizeof(*base->n_counts));
     /* Don't reset weights if logmath object hasn't changed. */
@@ -195,7 +227,7 @@ ngram_model_init(ngram_model_t *base,
         hash_table_empty(base->wid);
     else
         base->wid = hash_table_new(n_unigram, FALSE);
-    base->n_1g_alloc = base->n_words = n_unigram;
+    base->n_counts[0] = base->n_1g_alloc = base->n_words = n_unigram;
 
     return 0;
 }
@@ -260,6 +292,57 @@ ngram_model_free(ngram_model_t *model)
     return 0;
 }
 
+int
+ngram_model_casefold(ngram_model_t *model, int kase)
+{
+    int writable, i;
+    hash_table_t *new_wid;
+
+    /* Were word strings already allocated? */
+    writable = model->writable;
+    /* Either way, we are going to allocate some word strings. */
+    model->writable = TRUE;
+
+    /* And, don't forget, we need to rebuild the word to unigram ID
+     * mapping. */
+    new_wid = hash_table_new(model->n_words, FALSE);
+    for (i = 0; i < model->n_words; ++i) {
+        char *outstr;
+        if (writable) {
+            outstr = model->word_str[i];
+        }
+        else {
+            outstr = ckd_salloc(model->word_str[i]);
+        }
+        /* Don't case-fold <tags> or [classes] */
+        if (outstr[0] == '<' || outstr[0] == '[') {
+        }
+        else {
+            switch (kase) {
+            case NGRAM_UPPER:
+                ucase(outstr);
+                break;
+            case NGRAM_LOWER:
+                lcase(outstr);
+                break;
+            default:
+                ;
+            }
+        }
+        model->word_str[i] = outstr;
+
+        /* Now update the hash table.  We might have terrible
+         * collisions here, so warn about them. */
+        if (hash_table_enter_int32(new_wid, model->word_str[i], i) != i) {
+            E_WARN("Duplicate word in dictionary after conversion: %s\n",
+                   model->word_str[i]);
+        }
+    }
+    /* Swap out the hash table. */
+    hash_table_free(model->wid);
+    model->wid = new_wid;
+    return 0;
+}
 
 #ifdef HAVE_ICONV
 int
@@ -420,9 +503,7 @@ ngram_ng_score(ngram_model_t *model, int32 wid, int32 *history,
         ngram_class_t *lmclass = model->classes[NGRAM_CLASSID(wid)];
 
         class_weight = ngram_class_prob(lmclass, wid);
-        if (class_weight == 0) /* Yes, this is correct, because
-                                * log_zero is not available to
-                                * ngram_class_prob() */
+        if (class_weight == 1) /* Meaning, not found in class. */
             return model->log_zero;
         wid = lmclass->tag_wid;
     }
@@ -470,7 +551,9 @@ ngram_score(ngram_model_t *model, const char *word, ...)
 int32
 ngram_tg_score(ngram_model_t *model, int32 w3, int32 w2, int32 w1, int32 *n_used)
 {
-    int32 hist[2] = { w2, w1 };
+    int32 hist[2];
+    hist[0] = w2;
+    hist[1] = w1;
     return ngram_ng_score(model, w3, hist, 2, n_used);
 }
 
@@ -496,7 +579,7 @@ ngram_ng_prob(ngram_model_t *model, int32 wid, int32 *history,
         ngram_class_t *lmclass = model->classes[NGRAM_CLASSID(wid)];
 
         class_weight = ngram_class_prob(lmclass, wid);
-        if (class_weight == model->log_zero)
+        if (class_weight == 1) /* Meaning, not found in class. */
             return class_weight;
         wid = lmclass->tag_wid;
     }
@@ -589,6 +672,98 @@ ngram_model_get_counts(ngram_model_t *model)
   return NULL;
 }
 
+void
+ngram_iter_init(ngram_iter_t *itor, ngram_model_t *model,
+                int m, int successor)
+{
+    itor->model = model;
+    itor->wids = ckd_calloc(model->n, sizeof(*itor->wids));
+    itor->m = m;
+    itor->successor = successor;
+}
+
+ngram_iter_t *
+ngram_model_mgrams(ngram_model_t *model, int m)
+{
+    ngram_iter_t *itor;
+    /* The fact that m=n-1 is not exactly obvious.  Prevent accidents. */
+    if (m >= model->n)
+        return NULL;
+    if (model->funcs->mgrams == NULL)
+        return NULL;
+    itor = (*model->funcs->mgrams)(model, m);
+    return itor;
+}
+
+ngram_iter_t *
+ngram_iter(ngram_model_t *model, const char *word, ...)
+{
+    va_list history;
+    const char *hword;
+    int32 *histid;
+    int32 n_hist;
+    ngram_iter_t *itor;
+
+    va_start(history, word);
+    n_hist = 0;
+    while ((hword = va_arg(history, const char *)) != NULL)
+        ++n_hist;
+    va_end(history);
+
+    histid = ckd_calloc(n_hist, sizeof(*histid));
+    va_start(history, word);
+    n_hist = 0;
+    while ((hword = va_arg(history, const char *)) != NULL) {
+        histid[n_hist] = ngram_wid(model, hword);
+        ++n_hist;
+    }
+    va_end(history);
+
+    itor = ngram_ng_iter(model, ngram_wid(model, word), histid, n_hist);
+    ckd_free(histid);
+    return itor;
+}
+
+ngram_iter_t *
+ngram_ng_iter(ngram_model_t *model, int32 wid, int32 *history, int32 n_hist)
+{
+    if (n_hist >= model->n)
+        return NULL;
+    if (model->funcs->iter == NULL)
+        return NULL;
+    return (*model->funcs->iter)(model, wid, history, n_hist);
+}
+
+ngram_iter_t *
+ngram_iter_successors(ngram_iter_t *itor)
+{
+    /* Stop when we are at the highest order N-Gram. */
+    if (itor->m == itor->model->n - 1)
+        return NULL;
+    return (*itor->model->funcs->successors)(itor);
+}
+
+int32 const *
+ngram_iter_get(ngram_iter_t *itor,
+               int32 *out_score,
+               int32 *out_bowt)
+{
+    return (*itor->model->funcs->iter_get)(itor, out_score, out_bowt);
+}
+
+ngram_iter_t *
+ngram_iter_next(ngram_iter_t *itor)
+{
+    return (*itor->model->funcs->iter_next)(itor);
+}
+
+void
+ngram_iter_free(ngram_iter_t *itor)
+{
+    ckd_free(itor->wids);
+    (*itor->model->funcs->iter_free)(itor);
+}
+
 int32
 ngram_wid(ngram_model_t *model, const char *word)
 {
@@ -883,7 +1058,7 @@ ngram_class_prob(ngram_class_t *lmclass, int32 wid)
         while (hash != -1 && lmclass->nword_hash[hash].wid != wid)
             hash = lmclass->nword_hash[hash].next;
         if (hash == -1)
-            return 0;
+            return 1;
         return lmclass->nword_hash[hash].prob1;
     }
     else {
