@@ -49,6 +49,7 @@
 
 #include <string.h>
 #include <limits.h>
+#include <assert.h>
 
 static ngram_funcs_t ngram_model_arpa_funcs;
 
@@ -56,109 +57,29 @@ static ngram_funcs_t ngram_model_arpa_funcs;
 #define FIRST_BG(m,u)		((m)->lm3g.unigrams[u].bigrams)
 #define FIRST_TG(m,b)		(TSEG_BASE((m),(b))+((m)->lm3g.bigrams[b].trigrams))
 
-/*
- * Initialize sorted list with the 0-th entry = MIN_PROB_F, which may be needed
- * to replace spurious values in the Darpa LM file.
- */
-static void
-init_sorted_list(sorted_list_t * l)
-{
-    /* FIXME FIXME FIXME: Fixed size array!??! */
-    l->list = ckd_calloc(MAX_SORTED_ENTRIES,
-                         sizeof(sorted_entry_t));
-    l->list[0].val.l = INT_MIN;
-    l->list[0].lower = 0;
-    l->list[0].higher = 0;
-    l->free = 1;
-}
-
-static void
-free_sorted_list(sorted_list_t * l)
-{
-    free(l->list);
-}
-
-static lmprob_t *
-vals_in_sorted_list(sorted_list_t * l)
-{
-    lmprob_t *vals;
-    int32 i;
-
-    vals = ckd_calloc(l->free, sizeof(lmprob_t));
-    for (i = 0; i < l->free; i++)
-        vals[i] = l->list[i].val;
-    return (vals);
-}
-
-static int32
-sorted_id(sorted_list_t * l, int32 *val)
-{
-    int32 i = 0;
-
-    for (;;) {
-        if (*val == l->list[i].val.l)
-            return (i);
-        if (*val < l->list[i].val.l) {
-            if (l->list[i].lower == 0) {
-                if (l->free >= MAX_SORTED_ENTRIES) {
-                    /* Make the best of a bad situation. */
-                    E_WARN("sorted list overflow (%d => %d)\n",
-                           *val, l->list[i].val.l);
-                    return i;
-                }
-
-                l->list[i].lower = l->free;
-                (l->free)++;
-                i = l->list[i].lower;
-                l->list[i].val.l = *val;
-                return (i);
-            }
-            else
-                i = l->list[i].lower;
-        }
-        else {
-            if (l->list[i].higher == 0) {
-                if (l->free >= MAX_SORTED_ENTRIES) {
-                    /* Make the best of a bad situation. */
-                    E_WARN("sorted list overflow (%d => %d)\n",
-                           *val, l->list[i].val);
-                    return i;
-                }
-
-                l->list[i].higher = l->free;
-                (l->free)++;
-                i = l->list[i].higher;
-                l->list[i].val.l = *val;
-                return (i);
-            }
-            else
-                i = l->list[i].higher;
-        }
-    }
-}
-
 /*
  * Read and return #unigrams, #bigrams, #trigrams as stated in input file.
  */
 static int
-ReadNgramCounts(FILE * fp, int32 * n_ug, int32 * n_bg, int32 * n_tg)
+ReadNgramCounts(lineiter_t **li, int32 * n_ug, int32 * n_bg, int32 * n_tg)
 {
-    char string[256];
     int32 ngram, ngram_cnt;
 
     /* skip file until past the '\data\' marker */
-    do
-        fgets(string, sizeof(string), fp);
-    while ((strcmp(string, "\\data\\\n") != 0) && (!feof(fp)));
-
-    if (strcmp(string, "\\data\\\n") != 0) {
+    while (*li) {
+        string_trim((*li)->buf, STRING_BOTH);
+        if (strcmp((*li)->buf, "\\data\\") == 0)
+            break;
+        *li = lineiter_next(*li);
+    }
+    if (*li == NULL || strcmp((*li)->buf, "\\data\\") != 0) {
         E_ERROR("No \\data\\ mark in LM file\n");
         return -1;
     }
 
     *n_ug = *n_bg = *n_tg = 0;
-    while (fgets(string, sizeof(string), fp) != NULL) {
-        if (sscanf(string, "ngram %d=%d", &ngram, &ngram_cnt) != 2)
+    while ((*li = lineiter_next(*li))) {
+        if (sscanf((*li)->buf, "ngram %d=%d", &ngram, &ngram_cnt) != 2)
             break;
         switch (ngram) {
         case 1:
@@ -175,10 +96,21 @@ ReadNgramCounts(FILE * fp, int32 * n_ug, int32 * n_bg, int32 * n_tg)
             return -1;
         }
     }
+    if (*li == NULL) {
+        E_ERROR("EOF while reading ngram counts\n");
+        return -1;
+    }
 
-    /* Position file to just after the unigrams header '\1-grams:\' */
-    while ((strcmp(string, "\\1-grams:\n") != 0) && (!feof(fp)))
-        fgets(string, sizeof(string), fp);
+    /* Position iterator to the unigrams header '\1-grams:\' */
+    while ((*li = lineiter_next(*li))) {
+        string_trim((*li)->buf, STRING_BOTH);
+        if (strcmp((*li)->buf, "\\1-grams:") == 0)
+            break;
+    }
+    if (*li == NULL) {
+        E_ERROR_SYSTEM("Failed to read \\1-grams: mark");
+        return -1;
+    }
 
     /* Check counts;  NOTE: #trigrams *CAN* be 0 */
     if ((*n_ug <= 0) || (*n_bg <= 0) || (*n_tg < 0)) {
@@ -189,30 +121,32 @@ ReadNgramCounts(FILE * fp, int32 * n_ug, int32 * n_bg, int32 * n_tg)
 }
 
 /*
- * Read in the unigrams from given file into the LM structure model.  On
- * entry to this procedure, the file pointer is positioned just after the
+ * Read in the unigrams from given file into the LM structure model.
+ * On entry to this procedure, the iterator is positioned to the
  * header line '\1-grams:'.
  */
 static int
-ReadUnigrams(FILE * fp, ngram_model_arpa_t * model)
+ReadUnigrams(lineiter_t **li, ngram_model_arpa_t * model)
 {
     ngram_model_t *base = &model->base;
-    char string[256];
     int32 wcnt;
     float p1;
 
     E_INFO("Reading unigrams\n");
 
     wcnt = 0;
-    while ((fgets(string, sizeof(string), fp) != NULL) &&
-           (strcmp(string, "\\2-grams:\n") != 0)) {
+    while ((*li = lineiter_next(*li))) {
         char *wptr[3], *name;
         float32 bo_wt = 0.0f;
         int n;
 
-        if ((n = str2words(string, wptr, 3)) < 2) {
-            if (string[0] != '\n')
-                E_WARN("Format error; unigram ignored: %s\n", string);
+        string_trim((*li)->buf, STRING_BOTH);
+        if (strcmp((*li)->buf, "\\2-grams:") == 0)
+            break;
+
+        if ((n = str2words((*li)->buf, wptr, 3)) < 2) {
+            if ((*li)->buf[0] != '\0')
+                E_WARN("Format error; unigram ignored: %s\n", (*li)->buf);
             continue;
         }
         else {
@@ -242,6 +176,7 @@ ReadUnigrams(FILE * fp, ngram_model_arpa_t * model)
         E_WARN("lm_t.ucount(%d) != #unigrams read(%d)\n",
                base->n_counts[0], wcnt);
         base->n_counts[0] = wcnt;
+        base->n_words = wcnt;
     }
     return 0;
 }
@@ -250,10 +185,9 @@ ReadUnigrams(FILE * fp, ngram_model_arpa_t * model)
  * Read bigrams from given file into given model structure.
  */
 static int
-ReadBigrams(FILE * fp, ngram_model_arpa_t * model)
+ReadBigrams(lineiter_t **li, ngram_model_arpa_t * model)
 {
     ngram_model_t *base = &model->base;
-    char string[1024];
     int32 w1, w2, prev_w1, bgcount;
     bigram_t *bgptr;
 
@@ -263,15 +197,16 @@ ReadBigrams(FILE * fp, ngram_model_arpa_t * model)
     bgptr = model->lm3g.bigrams;
     prev_w1 = -1;
 
-    while (fgets(string, sizeof(string), fp) != NULL) {
+    while ((*li = lineiter_next(*li))) {
         float32 p, bo_wt = 0.0f;
         int32 p2, bo_wt2;
         char *wptr[4], *word1, *word2;
         int n;
 
+        string_trim((*li)->buf, STRING_BOTH);
         wptr[3] = NULL;
-        if ((n = str2words(string, wptr, 4)) < 3) {
-            if (string[0] != '\n')
+        if ((n = str2words((*li)->buf, wptr, 4)) < 3) {
+            if ((*li)->buf[0] != '\0')
                 break;
             continue;
         }
@@ -322,7 +257,6 @@ ReadBigrams(FILE * fp, ngram_model_arpa_t * model)
                 model->lm3g.unigrams[prev_w1].bigrams = bgcount;
             prev_w1 = w1;
         }
-
         bgcount++;
         bgptr++;
 
@@ -330,9 +264,9 @@ ReadBigrams(FILE * fp, ngram_model_arpa_t * model)
             E_INFOCONT(".");
         }
     }
-    if ((strcmp(string, "\\end\\") != 0)
-        && (strcmp(string, "\\3-grams:") != 0)) {
-        E_ERROR("Bad bigram: %s\n", string);
+    if (*li == NULL || ((strcmp((*li)->buf, "\\end\\") != 0)
+                        && (strcmp((*li)->buf, "\\3-grams:") != 0))) {
+        E_ERROR("Bad bigram: %s\n", (*li)->buf);
         return -1;
     }
 
@@ -346,10 +280,9 @@ ReadBigrams(FILE * fp, ngram_model_arpa_t * model)
  * Very similar to ReadBigrams.
  */
 static int
-ReadTrigrams(FILE * fp, ngram_model_arpa_t * model)
+ReadTrigrams(lineiter_t **li, ngram_model_arpa_t * model)
 {
     ngram_model_t *base = &model->base;
-    char string[1024];
     int32 i, w1, w2, w3, prev_w1, prev_w2, tgcount, prev_bg, bg, endbg;
     int32 seg, prev_seg, prev_seg_lastbg;
     trigram_t *tgptr;
@@ -364,13 +297,14 @@ ReadTrigrams(FILE * fp, ngram_model_arpa_t * model)
     prev_bg = -1;
     prev_seg = -1;
 
-    while (fgets(string, sizeof(string), fp) != NULL) {
+    while ((*li = lineiter_next(*li))) {
         float32 p;
         int32 p3;
         char *wptr[4], *word1, *word2, *word3;
 
-        if (str2words(string, wptr, 4) != 4) {
-            if (string[0] != '\n')
+        string_trim((*li)->buf, STRING_BOTH);
+        if (str2words((*li)->buf, wptr, 4) != 4) {
+            if ((*li)->buf[0] != '\0')
                 break;
             continue;
         }
@@ -423,7 +357,7 @@ ReadTrigrams(FILE * fp, ngram_model_arpa_t * model)
             bgptr = model->lm3g.bigrams + bg;
             for (; (bg < endbg) && (bgptr->wid != w2); bg++, bgptr++);
             if (bg >= endbg) {
-                E_ERROR("Missing bigram for trigram: %s", string);
+                E_ERROR("Missing bigram for trigram: %s", (*li)->buf);
                 return -1;
             }
 
@@ -480,8 +414,8 @@ ReadTrigrams(FILE * fp, ngram_model_arpa_t * model)
             E_INFOCONT(".");
         }
     }
-    if (strcmp(string, "\\end\\") != 0) {
-        E_ERROR("Bad trigram: %s\n", string);
+    if (*li == NULL || strcmp((*li)->buf, "\\end\\") != 0) {
+        E_ERROR("Bad trigram: %s\n", (*li)->buf);
         return -1;
     }
 
@@ -517,6 +451,7 @@ ngram_model_arpa_read(cmd_ln_t *config,
 		      const char *file_name,
 		      logmath_t *lmath)
 {
+    lineiter_t *li;
     FILE *fp;
     int32 is_pipe;
     int32 n_unigram;
@@ -530,9 +465,11 @@ ngram_model_arpa_read(cmd_ln_t *config,
         E_ERROR("File %s not found\n", file_name);
         return NULL;
     }
+    li = lineiter_start(fp);
  
     /* Read #unigrams, #bigrams, #trigrams from file */
-    if (ReadNgramCounts(fp, &n_unigram, &n_bigram, &n_trigram) == -1) {
+    if (ReadNgramCounts(&li, &n_unigram, &n_bigram, &n_trigram) == -1) {
+        lineiter_free(li);
         fclose_comp(fp, is_pipe);
         return NULL;
     }
@@ -570,7 +507,7 @@ ngram_model_arpa_read(cmd_ln_t *config,
             ckd_calloc((n_bigram + 1) / BG_SEG_SZ + 1,
                        sizeof(int32));
     }
-    if (ReadUnigrams(fp, model) == -1) {
+    if (ReadUnigrams(&li, model) == -1) {
         fclose_comp(fp, is_pipe);
         ngram_model_free(base);
         return NULL;
@@ -581,7 +518,7 @@ ngram_model_arpa_read(cmd_ln_t *config,
     if (base->n_counts[2] > 0)
         init_sorted_list(&model->sorted_bo_wt2);
 
-    if (ReadBigrams(fp, model) == -1) {
+    if (ReadBigrams(&li, model) == -1) {
         fclose_comp(fp, is_pipe);
         ngram_model_free(base);
         return NULL;
@@ -603,7 +540,7 @@ ngram_model_arpa_read(cmd_ln_t *config,
 
         init_sorted_list(&model->sorted_prob3);
 
-        if (ReadTrigrams(fp, model) == -1) {
+        if (ReadTrigrams(&li, model) == -1) {
             fclose_comp(fp, is_pipe);
             ngram_model_free(base);
             return NULL;
@@ -622,6 +559,7 @@ ngram_model_arpa_read(cmd_ln_t *config,
         model->lm3g.le = listelem_alloc_init(sizeof(tginfo_t));
     }
 
+    lineiter_free(li);
     fclose_comp(fp, is_pipe);
     return base;
 }
@@ -630,7 +568,47 @@ int
 ngram_model_arpa_write(ngram_model_t *model,
 		       const char *file_name)
 {
-    return -1;
+    ngram_iter_t *itor;
+    FILE *fh;
+    int i;
+
+    if ((fh = fopen(file_name, "w")) == NULL) {
+        E_ERROR_SYSTEM("Failed to open %s for writing", file_name);
+        return -1;
+    }
+    fprintf(fh, "This is an ARPA-format language model file, generated by CMU Sphinx\n");
+
+    /* The ARPA format doesn't require any extra information that
+     * N-Gram iterators can't give us, so this is very
+     * straightforward compared with DMP writing. */
+
+    /* Write N-gram counts. */
+    fprintf(fh, "\\data\\\n");
+    for (i = 0; i < model->n; ++i) {
+        fprintf(fh, "ngram %d=%d\n", i+1, model->n_counts[i]);
+    }
+
+    /* Write N-grams */
+    for (i = 0; i < model->n; ++i) {
+        fprintf(fh, "\n\\%d-grams:\n", i + 1);
+        for (itor = ngram_model_mgrams(model, i); itor; itor = ngram_iter_next(itor)) {
+            int32 const *wids;
+            int32 score, bowt;
+            int j;
+
+            wids = ngram_iter_get(itor, &score, &bowt);
+            fprintf(fh, "%.4f ", logmath_log_to_log10(model->lmath, score));
+            for (j = 0; j <= i; ++j) {
+                assert(wids[j] < model->n_counts[0]);
+                fprintf(fh, "%s ", model->word_str[wids[j]]);
+            }
+            if (i < model->n-1)
+                fprintf(fh, "%.4f", logmath_log_to_log10(model->lmath, bowt));
+            fprintf(fh, "\n");
+        }
+    }
+    fprintf(fh, "\n\\end\\\n");
+    return fclose(fh);
 }
 
 static int
@@ -642,231 +620,11 @@ ngram_model_arpa_apply_weights(ngram_model_t *base, float32 lw,
     return 0;
 }
 
-/* Locate a specific bigram within a bigram list */
-#define BINARY_SEARCH_THRESH	16
-static int32
-find_bg(bigram_t * bg, int32 n, int32 w)
-{
-    int32 i, b, e;
-
-    /* Binary search until segment size < threshold */
-    b = 0;
-    e = n;
-    while (e - b > BINARY_SEARCH_THRESH) {
-        i = (b + e) >> 1;
-        if ((int32)bg[i].wid < w)
-            b = i + 1;
-        else if ((int32)bg[i].wid > w)
-            e = i;
-        else
-            return i;
-    }
-
-    /* Linear search within narrowed segment */
-    for (i = b; (i < e) && (bg[i].wid != w); i++);
-    return ((i < e) ? i : -1);
-}
-
-static int32
-lm3g_bg_score(ngram_model_arpa_t *model, int32 lw1,
-              int32 lw2, int32 *n_used)
-{
-    int32 i, n, b, score;
-    bigram_t *bg;
-
-    if (lw1 < 0) {
-        *n_used = 1;
-        return model->lm3g.unigrams[lw2].prob1.l;
-    }
-
-    b = FIRST_BG(model, lw1);
-    n = FIRST_BG(model, lw1 + 1) - b;
-    bg = model->lm3g.bigrams + b;
-
-    if ((i = find_bg(bg, n, lw2)) >= 0) {
-        /* Access mode = bigram */
-        *n_used = 2;
-        score = model->lm3g.prob2[bg[i].prob2].l;
-    }
-    else {
-        /* Access mode = unigram */
-        *n_used = 1;
-        score = model->lm3g.unigrams[lw1].bo_wt1.l + model->lm3g.unigrams[lw2].prob1.l;
-    }
-
-    return (score);
-}
-
-static void
-load_tginfo(ngram_model_arpa_t *model, int32 lw1, int32 lw2)
-{
-    int32 i, n, b, t;
-    bigram_t *bg;
-    tginfo_t *tginfo;
-
-    /* First allocate space for tg information for bg lw1,lw2 */
-    tginfo = (tginfo_t *) listelem_malloc(model->lm3g.le);
-    tginfo->w1 = lw1;
-    tginfo->tg = NULL;
-    tginfo->next = model->lm3g.tginfo[lw2];
-    model->lm3g.tginfo[lw2] = tginfo;
-
-    /* Locate bigram lw1,lw2 */
-    b = model->lm3g.unigrams[lw1].bigrams;
-    n = model->lm3g.unigrams[lw1 + 1].bigrams - b;
-    bg = model->lm3g.bigrams + b;
-
-    if ((n > 0) && ((i = find_bg(bg, n, lw2)) >= 0)) {
-        tginfo->bowt = model->lm3g.bo_wt2[bg[i].bo_wt2].l;
-
-        /* Find t = Absolute first trigram index for bigram lw1,lw2 */
-        b += i;                 /* b = Absolute index of bigram lw1,lw2 on disk */
-        t = FIRST_TG(model, b);
-
-        tginfo->tg = model->lm3g.trigrams + t;
-
-        /* Find #tg for bigram w1,w2 */
-        tginfo->n_tg = FIRST_TG(model, b + 1) - t;
-    }
-    else {                      /* No bigram w1,w2 */
-        tginfo->bowt = 0;
-        tginfo->n_tg = 0;
-    }
-}
-
-/* Similar to find_bg */
-static int32
-find_tg(trigram_t * tg, int32 n, int32 w)
-{
-    int32 i, b, e;
-
-    b = 0;
-    e = n;
-    while (e - b > BINARY_SEARCH_THRESH) {
-        i = (b + e) >> 1;
-        if ((int32)tg[i].wid < w)
-            b = i + 1;
-        else if ((int32)tg[i].wid > w)
-            e = i;
-        else
-            return i;
-    }
-
-    for (i = b; (i < e) && (tg[i].wid != w); i++);
-    return ((i < e) ? i : -1);
-}
-
-static int32
-lm3g_tg_score(ngram_model_arpa_t *model, int32 lw1,
-              int32 lw2, int32 lw3, int32 *n_used)
-{
-    ngram_model_t *base = &model->base;
-    int32 i, n, score;
-    trigram_t *tg;
-    tginfo_t *tginfo, *prev_tginfo;
-
-    if ((base->n < 3) || (lw1 < 0))
-        return (lm3g_bg_score(model, lw2, lw3, n_used));
-
-    prev_tginfo = NULL;
-    for (tginfo = model->lm3g.tginfo[lw2]; tginfo; tginfo = tginfo->next) {
-        if (tginfo->w1 == lw1)
-            break;
-        prev_tginfo = tginfo;
-    }
-
-    if (!tginfo) {
-        load_tginfo(model, lw1, lw2);
-        tginfo = model->lm3g.tginfo[lw2];
-    }
-    else if (prev_tginfo) {
-        prev_tginfo->next = tginfo->next;
-        tginfo->next = model->lm3g.tginfo[lw2];
-        model->lm3g.tginfo[lw2] = tginfo;
-    }
-
-    tginfo->used = 1;
-
-    /* Trigrams for w1,w2 now pointed to by tginfo */
-    n = tginfo->n_tg;
-    tg = tginfo->tg;
-    if ((i = find_tg(tg, n, lw3)) >= 0) {
-        /* Access mode = trigram */
-        *n_used = 3;
-        score = model->lm3g.prob3[tg[i].prob3].l;
-    }
-    else {
-        score = tginfo->bowt + lm3g_bg_score(model, lw2, lw3, n_used);
-    }
-
-    return (score);
-}
-
-static int32
-ngram_model_arpa_score(ngram_model_t *base, int32 wid,
-                       int32 *history, int32 n_hist,
-                       int32 *n_used)
-{
-    ngram_model_arpa_t *model = (ngram_model_arpa_t *)base;
-
-    switch (n_hist) {
-    case 0:
-        /* Access mode: unigram */
-        *n_used = 1;
-        return model->lm3g.unigrams[wid].prob1.l;
-    case 1:
-        return lm3g_bg_score(model, history[0], wid, n_used);
-    case 2:
-    default:
-        /* Anything greater than 2 is the same as a trigram for now. */
-        return lm3g_tg_score(model, history[1], history[0], wid, n_used);
-    }
-}
-
-static int32
-ngram_model_arpa_raw_score(ngram_model_t *base, int32 wid,
-                           int32 *history, int32 n_hist,
-                           int32 *n_used)
-{
-    ngram_model_arpa_t *model = (ngram_model_arpa_t *)base;
-    int32 score;
-
-    switch (n_hist) {
-    case 0:
-        /* Access mode: unigram */
-        *n_used = 1;
-        /* Undo insertion penalty. */
-        score = model->lm3g.unigrams[wid].prob1.l - base->log_wip;
-        /* Undo language weight. */
-        score = (int32)(score / base->lw);
-        /* Undo unigram interpolation */
-        if (strcmp(base->word_str[wid], "<s>") != 0) { /* FIXME: configurable start_sym */
-            score = logmath_log(base->lmath,
-                                logmath_exp(base->lmath, score)
-                                - logmath_exp(base->lmath, 
-                                              base->log_uniform + base->log_uniform_weight));
-        }
-        return score;
-    case 1:
-        score = lm3g_bg_score(model, history[0], wid, n_used);
-        break;
-    case 2:
-    default:
-        /* Anything greater than 2 is the same as a trigram for now. */
-        score = lm3g_tg_score(model, history[1], history[0], wid, n_used);
-        break;
-    }
-    /* FIXME (maybe): This doesn't undo unigram weighting in backoff cases. */
-    return (int32)((score - base->log_wip) / base->lw);
-}
-
-static int32
-ngram_model_arpa_add_ug(ngram_model_t *base,
-                        int32 wid, int32 lweight)
-{
-    ngram_model_arpa_t *model = (ngram_model_arpa_t *)base;
-    return lm3g_add_ug(base, &model->lm3g, wid, lweight);
-}
+/* Lousy "templating" for things that are largely the same in DMP and
+ * ARPA models, except for the bigram and trigram types and some
+ * names. */
+#define NGRAM_MODEL_TYPE ngram_model_arpa_t
+#include "lm3g_templates.c"
 
 static void
 ngram_model_arpa_free(ngram_model_t *base)
@@ -882,18 +640,17 @@ ngram_model_arpa_free(ngram_model_t *base)
     ckd_free(model->lm3g.tseg_base);
 }
 
-static void
-ngram_model_arpa_flush(ngram_model_t *base)
-{
-    ngram_model_arpa_t *model = (ngram_model_arpa_t *)base;
-    lm3g_tginfo_reset(base, &model->lm3g);
-}
-
 static ngram_funcs_t ngram_model_arpa_funcs = {
     ngram_model_arpa_free,          /* free */
     ngram_model_arpa_apply_weights, /* apply_weights */
-    ngram_model_arpa_score,         /* score */
-    ngram_model_arpa_raw_score,     /* raw_score */
-    ngram_model_arpa_add_ug,        /* add_ug */
-    ngram_model_arpa_flush          /* flush */
+    lm3g_template_score,            /* score */
+    lm3g_template_raw_score,        /* raw_score */
+    lm3g_template_add_ug,           /* add_ug */
+    lm3g_template_flush,            /* flush */
+    lm3g_template_iter,             /* iter */
+    lm3g_template_mgrams,           /* mgrams */
+    lm3g_template_successors,       /* successors */
+    lm3g_template_iter_get,         /* iter_get */
+    lm3g_template_iter_next,        /* iter_next */
+    lm3g_template_iter_free         /* iter_free */
 };
