@@ -135,7 +135,7 @@ ngram_model_set_init(cmd_ln_t *config,
     /* Do consistency checking on the models.  They must all use the
      * same logbase and shift. */
     lmath = models[0]->lmath;
-    for (i = 0; i < n_models; ++i) {
+    for (i = 1; i < n_models; ++i) {
         if (logmath_get_base(models[i]->lmath) != logmath_get_base(lmath)
             || logmath_get_shift(models[i]->lmath) != logmath_get_shift(lmath)) {
             E_ERROR("Log-math parameters don't match, will not create LM set\n");
@@ -276,7 +276,6 @@ ngram_model_set_read(cmd_ln_t *config,
                     void *val;
                     classdef_t *classdef;
 
-                    E_INFO("Adding class '%s'\n", str);
                     if (hash_table_lookup(classes, str, &val) == -1) {
                         E_ERROR("Unknown class %s in control file\n", str);
                         goto error_out;
@@ -287,6 +286,8 @@ ngram_model_set_read(cmd_ln_t *config,
                                               classdef->n_words) < 0) {
                         goto error_out;
                     }
+                    E_INFO("Added class %s containing %d words\n",
+                           str, classdef->n_words);
                 }
                 if (strcmp(str, "}") != 0) {
                     E_ERROR("Unexpected EOF in %s\n", lmctlfile);
@@ -460,7 +461,7 @@ ngram_model_set_current_wid(ngram_model_t *base,
     if (set->cur == -1 || set_wid >= base->n_words)
         return NGRAM_INVALID_WID;
     else
-        return set->widmap[set->cur][set_wid];
+        return set->widmap[set_wid][set->cur];
 }
 
 int32
@@ -474,7 +475,7 @@ ngram_model_set_known_wid(ngram_model_t *base,
     else if (set->cur == -1) {
         int32 i;
         for (i = 0; i < set->n_models; ++i) {
-            if (set->widmap[i][set_wid] != ngram_unknown_wid(set->lms[i]))
+            if (set->widmap[set_wid][i] != ngram_unknown_wid(set->lms[i]))
                 return TRUE;
         }
         return FALSE;
@@ -541,7 +542,7 @@ ngram_model_set_add(ngram_model_t *base,
     }
 
     /* Renormalize the interpolation weights. */
-    fprob = weight * 1.0f / set->n_models;
+    fprob = weight * 1.0 / set->n_models;
     set->lweights = ckd_realloc(set->lweights,
                                 set->n_models * sizeof(*set->lweights));
     set->lweights[set->n_models - 1] = logmath_log(base->lmath, fprob);
@@ -594,7 +595,7 @@ ngram_model_set_remove(ngram_model_t *base,
 
     /* Renormalize the interpolation weights by scaling them by
      * 1/(1-fprob) */
-    fprob = (float32)logmath_exp(base->lmath, set->lweights[lmidx]);
+    fprob = logmath_exp(base->lmath, set->lweights[lmidx]);
     scale = logmath_log(base->lmath, 1.0 - fprob);
 
     /* Remove it from the array of lms, renormalize remaining weights,
@@ -799,7 +800,7 @@ ngram_model_set_add_ug(ngram_model_t *base,
             if (newwid[i] == NGRAM_INVALID_WID) {
                 /* Add it to the submodel. */
                 newwid[i] = ngram_model_add_word(set->lms[i], base->word_str[wid],
-                                                 (float32)logmath_exp(base->lmath, lweight));
+                                                 logmath_exp(base->lmath, lweight));
                 if (newwid[i] == NGRAM_INVALID_WID) {
                     ckd_free(newwid);
                     return base->log_zero;
