@@ -50,7 +50,7 @@
 /*
  * Platform-specific parts: threads, mutexes, and signals.
  */
-#ifdef _WIN32
+#if (defined(_WIN32) || defined(__CYGWIN__)) && !defined(__SYMBIAN32__)
 #define _WIN32_WINNT 0x0400
 #include <windows.h>
 
@@ -149,13 +149,14 @@ cond_timed_wait(HANDLE cond, int sec, int nsec)
     return rv;
 }
 
+/* Silvio Moioli: updated to use Unicode */
 sbevent_t *
 sbevent_init(void)
 {
     sbevent_t *evt;
 
     evt = ckd_calloc(1, sizeof(*evt));
-    evt->evt = CreateEvent(NULL, FALSE, FALSE, NULL);
+    evt->evt = CreateEventW(NULL, FALSE, FALSE, NULL);
     if (evt->evt == NULL) {
         ckd_free(evt);
         return NULL;
@@ -229,7 +230,7 @@ sbmsgq_init(size_t depth)
 
     msgq = ckd_calloc(1, sizeof(*msgq));
     msgq->depth = depth;
-    msgq->evt = CreateEvent(NULL, FALSE, FALSE, NULL);
+    msgq->evt = CreateEventW(NULL, FALSE, FALSE, NULL);
     if (msgq->evt == NULL) {
         ckd_free(msgq);
         return NULL;
@@ -513,7 +514,7 @@ sbmsgq_send(sbmsgq_t *q, size_t len, void const *data)
         size_t len1 = q->depth - in;
         memcpy(q->data + in, data, len1);
         q->nbytes += len1;
-        data += len1;
+        data = (char const *)data + len1;
         len -= len1;
         in = 0;
     }
